@@ -1,101 +1,176 @@
 ###############################
 #mcfs
 ###############################
-mcfs <- function(formula, data,
-                        projections = 'auto',
-                        projectionSize = 'auto',
-                        featureFreq = 150,
-                        splits = 5,
-                        splitSetSize = 1000,
-                        balance = 'auto', 
-                        cutoffMethod = c("permutations", "criticalAngle", "kmeans", "mean"),
-                        cutoffPermutations = 20,
-                        buildID = TRUE,
-                        finalRuleset = TRUE,
-                        finalCV = TRUE,
-                        finalCVSetSize = 1000,
-                        finalCVRepetitions = 3,
-                        seed = NA,
-                        threadsNumber = 2)
+mcfs <- function(formula, data, 
+                 attrWeights = NULL,
+                 projections = 'auto',
+                 projectionSize = 'auto',
+                 featureFreq = 100,
+                 splits = 5,
+                 splitSetSize = 500,
+                 balance = 'auto', 
+                 cutoffMethod = c("permutations", "criticalAngle", "kmeans", "mean" , "contrast"),
+                 cutoffPermutations = 20,
+                 mode = 1,
+                 buildID = TRUE,
+                 finalRuleset = TRUE,
+                 finalCV = TRUE,
+                 finalCVSetSize = 1000,
+                 seed = NA,
+                 threadsNumber = 4)
 {
-  if(get.JavaVersion() < 1.6) 
-    stop("Java 6 or higher is needed for this package but is not available.")
-  
+  if(get.JavaVersion() < 7){
+    mystop("Java 7 or higher is needed for this package but not available.")
+    return(NULL)
+  }
   if(!cutoffMethod[1] %in% c("permutations", "criticalAngle", "kmeans", "mean")){
-    stop(paste0("Incorrect 'cutoffMethod' = ", cutoffMethod[1]))
+    mystop(paste0("Incorrect 'cutoffMethod' = ", cutoffMethod[1]))
+    return(NULL)
   }
   
-  start.time <- Sys.time()
   cat("Checking input data...\n")
-  label <- "input"
   coldef <- formula.prepare(formula, data)
   target <- coldef$target
   cols <- coldef$cols
 
-  data <- cbind(data[,cols, drop=FALSE], data[,target, drop=FALSE])
-
-  if(length(unique(data[,target])) == 1){
-    stop("Decision attribute contains only 1 value.")
+  #set up attrWeights
+  if(!is.null(attrWeights)){
+    if(length(attrWeights) != ncol(data)){
+      mystop(paste0("Size (ncol=", ncol(data), ") of input data.frame does not match to length of weights vector(",
+                length(attr(data, "attr_weights")),")"))
+      return(NULL)
+    }
+    if(!is.numeric(attrWeights)){
+      mystop("Parameter 'attrWeights' does not define numeric weights.")
+      return(NULL)
+    }
+    attrWeights <- matrix(attrWeights, nrow = 1, dimnames = list("1", colnames(data)))
   }
-  if(any(is.na(data[,target])) == T){
-    stop("Decision attribute contains NA values.")
+  #set up input data
+  data <- data[, c(cols, target), drop=FALSE]
+  attr(data, 'target') <- target
+  if(!is.null(attrWeights)){
+    attr(data, 'attr_weights') <- as.vector(attrWeights[1, c(cols, target)])
   }
   
-  tmp_dir <- tempdir()
+  #check if data is all right
+  if(check_data(data) == FALSE)
+    return(NULL)
 
+  #in any case replace slash
+  tmp_dir <- fix.path(tempdir())
+  
+  #set the label
+  #label <- paste0("input_", target)
+  label <- "input"
+  #set the config file
   config_file <- file.path(tmp_dir, "mcfs.run")
-  input_file_name <- paste0(label, ".adx")
+
   params <- default.params
-  params$inputFileName = input_file_name
-  params$inputFilesPATH = paste0(gsub("\\\\", .Platform$file.sep, tmp_dir), .Platform$file.sep)
-  params$resFilesPATH = paste0(gsub("\\\\", .Platform$file.sep, tmp_dir), .Platform$file.sep)
-  params$mcfs.projections = projections
-  params$mcfs.projectionSize = get.projectionSize(length(cols), projectionSize)
-  params$mcfs.splits = splits
-  params$mcfs.featureFreq = featureFreq
-  params$mcfs.balance = balance
-  params$mcfs.splitSetSize = splitSetSize
-  params$mcfs.cutoffPermutations = cutoffPermutations
-  params$mcfs.cutoffMethod = cutoffMethod[1]
-  params$mcfs.buildID = buildID
-  params$mcfs.finalRuleset = finalRuleset
-  params$mcfs.finalCV = finalCV
-  params$mcfs.finalCVSetSize = finalCVSetSize
-  params$mcfs.finalCVRepetitions = finalCVRepetitions
-  params$mcfs.threadsNumber = threadsNumber
+  params$inputFileName <- paste0(label, ".adh")
+  params$inputFilesPATH <- file.path(tmp_dir, 'rmcfs_data')
+  params$resFilesPATH <- file.path(tmp_dir, 'rmcfs_result')
+  params$mcfs.projections <- projections
+  params$mcfs.projectionSize <- projectionSize
+  #params$mcfs.projectionSize <- get.projectionSize(length(cols), projectionSize)
+  params$mcfs.splits <- splits
+  params$mcfs.featureFreq <- featureFreq
+  params$mcfs.balance <- balance
+  params$mcfs.splitSetSize <- splitSetSize
+  params$mcfs.cutoffPermutations <- cutoffPermutations
+  params$mcfs.cutoffMethod <- cutoffMethod[1]
+  params$mcfs.buildID <- buildID
+  params$mcfs.finalRuleset <- finalRuleset
+  params$mcfs.finalCV <- finalCV
+  params$mcfs.finalCVSetSize <- finalCVSetSize
+  params$mcfs.threadsNumber <- threadsNumber
+  params$mcfs.progressInterval <- 10
+
   if(is.numeric(seed)){
-    params$mcfs.seed = seed
+    params$mcfs.seed <- seed
   }else{
-    params$mcfs.seed = round(runif(1, 0, 2^24))
+    params$mcfs.seed <- round(runif(1, 0, 2^24))
+  }
+  
+  if(!params$mcfs.mode %in% c(1,2)){
+    params$mcfs.mode <- 1
+    warning("Parameter mode")
+  }else{
+    params$mcfs.mode <- mode
   }
 
+  dir.create(params$inputFilesPATH, showWarnings = FALSE)
+  dir.create(params$resFilesPATH, showWarnings = FALSE)
+  clean.dir(params$inputFilesPATH)
+  clean.dir(params$resFilesPATH)
+  
+  #for DEBUG switch these two to TRUE
+  #params$verbose <- T
+  #params$debug <- T
+  
   cat("Exporting params...\n")
+  #cat(paste0('File: ', config_file,'\n'))
   save.params.file(params, config_file)
   
   cat("Exporting input data...\n")
-  write.adx(data, file=file.path(tmp_dir, input_file_name), target=target)
-  
+  input_file <- file.path(params$inputFilesPATH, params$inputFileName)
+  #cat(paste0('File: ', input_file , '\n'))
+  write.adh(data, file = input_file, target = target)
+
+  start.time <- Sys.time()
   cat("Running MCFS-ID...\n")
   cdir <- getwd()
   setwd(libdir)
   #jri.prepare()
-  .jcall("dmLab/mcfs/MCFS", returnSig="V", "main",
-         .jarray(config_file))
+  .jcall("dmLab/mcfs/MCFS", returnSig="V", "main", .jarray(config_file))
   .jcheck(silent = FALSE)
   setwd(cdir)
+  end.time <- Sys.time()
+  exec_time <- end.time - start.time
   
   cat("Reading results...\n")
-  mcfsResult <- import.result(tmp_dir, label)
+  mcfsResult <- import.result(params$resFilesPATH, label)
+  mcfsResult$exec_time <- exec_time
+
+
+  myRI <- mcfsResult$RI[order(as.numeric(rownames(mcfsResult$RI))),]
+  # check if attributes have still the same name
+  data_features <- names(data)[!names(data) %in% attr(data, 'target')]
+  if(any(as.numeric(rownames(myRI)) != myRI$position)){
+    attr_changed <- data.frame(from = myRI$attribute[myRI$attribute != data_features], 
+                               to = data_features[myRI$attribute != data_features], stringsAsFactors = F)
+    if(nrow(attr_changed) > 0){
+      warning(paste0("In the MCFS-ID result the following names of attributes must be fixed/reset: '", paste0(attr_changed$from, collapse = "','"),"'"))
+      #fix ranking (RI) names
+      myRI$attribute[myRI$attribute != data_features] <- data_features[myRI$attribute != data_features]
+      mcfsResult$RI <- myRI[order(as.numeric(myRI$position)),]
+      
+      #fix ID-graph names - edge_a
+      mcfsResult$ID$edge_a <- sapply(mcfsResult$ID$edge_a, function(x, attrChanged){
+        change <- (x == attrChanged$from)
+        if(any(change)){
+          return(as.character(attrChanged$to[change]))
+        }else 
+          return(x)
+      },attrChanged = attr_changed)
+
+      #fix ID-graph names - edge_b
+      mcfsResult$ID$edge_b <- sapply(mcfsResult$ID$edge_b, function(x, attrChanged){
+        change <- (x == attrChanged$from)
+        if(any(change)){
+          return(as.character(attrChanged$to[change]))
+        }else 
+          return(x)
+      },attrChanged = attr_changed)
+    }
+  }
   
   #clean temp files
-  #cat("Cleaning temporary files...\n")
   tmp.files <- get.files.names(tmp_dir, ext=c('.zip','.run','.csv','.txt','.adx','.adh'), fullNames=T, recursive=F)
   if(length(tmp.files)>0)
     delete.files(tmp.files)
   
   cat("Done.\n")
-  end.time <- Sys.time()
-  mcfsResult$exec_time <- end.time - start.time
 
   class(mcfsResult) <- "mcfs"
   return(mcfsResult)
@@ -126,7 +201,8 @@ jri.prepare <- function(){
 formula.prepare <- function(formula, data) {
   target <- as.character(as.list(formula)[[2]])
   if(!target %in% names(data)){
-    stop(paste0("Target feature: '", target, "' does not exist in the data."))
+    mystop(paste0("Target feature: '", target, "' does not exist in the data."))
+    return(NULL)
   }
   
   #function model.frame crashes under LINUX
@@ -144,7 +220,8 @@ formula.prepare <- function(formula, data) {
   cols.missing <- cols[!var.exists]
   if(length(cols.missing)>0){
     cols.missing.text <- paste(cols.missing, collapse=", ")
-    stop(paste0("Features: '", cols.missing.text, "' does not exist in the data."))
+    mystop(paste0("Features: '", cols.missing.text, "' does not exist in the data."))
+    return(NULL)
   }
   retList <- list(target=target, cols=cols)
   return(retList)
@@ -155,11 +232,13 @@ formula.prepare <- function(formula, data) {
 ###############################
 default.params <- list(verbose = "false", debug = "false", mcfs.progressShow = "false",
                        inputFileName = "", inputFilesPATH = "", resFilesPATH = "",
-                       mcfs.projections = 'auto', mcfs.projectionSize = 'auto',
+                       mcfs.projections = 'auto', 
+                       mcfs.projectionSize = 'auto',
                        mcfs.projectionSizeMax = 1000,
                        mcfs.featureFreq = 100,
-                       mcfs.splits = 5, mcfs.splitRatio = 0.66,
-                       mcfs.splitSetSize = 1000,
+                       mcfs.splits = 5, 
+                       mcfs.splitRatio = 0.66,
+                       mcfs.splitSetSize = 500,
                        mcfs.balance = 'auto',
                        mcfs.mode = 1,
                        mcfs.cutoffMethod = "permutations",
@@ -171,10 +250,10 @@ default.params <- list(verbose = "false", debug = "false", mcfs.progressShow = "
                        mcfs.finalCV = "true",
                        mcfs.finalCVSetSize = 1000,
                        mcfs.finalCVRepetitions = 3,
-                       mcfs.model = "auto", mcfs.progressInterval = 10,
+                       mcfs.finalCVfolds = 10,
+                       mcfs.model = "auto", 
+                       mcfs.progressInterval = 10,
                        mcfs.u = 1, mcfs.v = 1,
-                       mcfs.contrastAttr = "false",
-                       mcfs.contrastAttrThreshold = 1,
                        mcfs.zipResult = F,
                        mcfs.seed = NA,
                        mcfs.threadsNumber = 4,
@@ -206,6 +285,48 @@ save.params.file <- function(params, config_file) {
   close(f)
 }
 
+###############################
+#check_data
+###############################
+check_data <- function(data)
+{
+  retVal <- TRUE
+  if(!inherits(data, "data.frame")){
+    retVal <- mystop("Input data object is not a data.frame.")
+  }
+  
+  if('target' %in% names(attributes(data))){
+    target <- attr(data, 'target')
+    if(!target %in% names(data)){
+      retVal <- mystop(paste0("Target attribute: '", target,"' does not exist in the input data."))
+    }
+  }else{
+    retVal <- mystop(paste0("Target attribute is not defined."))
+  }
+  if(length(unique(data[,target])) == 1){
+    retVal <- mystop("Target attribute contains only 1 value.")
+  }
+  if(any(is.na(data[,target])) == T){
+    retVal <- mystop("Target attribute contains NA values.")
+  }
+  if(any(startsWith(tolower(names(data)),"mcfs_contrast_attr")) == T){
+    retVal <- mystop("Attribute name that starts with 'mcfs_contrast_attr' is forbidden.")
+  }
+  #data.frame attributes
+  if( ('attr_weights' %in% names(attributes(data))) & (ncol(data) != length(attr(data, "attr_weights"))) ){
+    retVal <- mystop(paste0("Size (ncol=", ncol(data), ") of input data.frame does not match to length of weights vector(",
+                  length(attr(data, "attr_weights")),")"))
+  }
+  source_chars <- c(" ", "'", ",", "/", "|", "#", "-", "(", ")", "[", "]", "{", "}")
+  forb_attr <- string.detect(names(data), source_chars)
+  if(any(forb_attr)){
+    retVal <- mystop(paste0("The names of the following attributes: \n",paste0(names(data)[forb_attr], collapse = ",\n "),
+                            "\n contain forbidden characters. Please run fix.data() function before running mcfs()."))
+  }
+  
+  return(retVal)
+}
+
 ###############################
 #fix.matrix
 ###############################
@@ -213,7 +334,7 @@ fix.matrix <- function(m, na.char = '?'){
   m[is.na(m)] <- na.char
   m[is.infinite(m)] <- na.char
   if(class(m[1,1])=="character"){
-    m <- string.trim(m)
+    m[] <- string.trim(m)
     m[m=="Inf"] <- na.char
     m[m=="-Inf"] <- na.char
   }  
@@ -232,8 +353,9 @@ fix.matrix <- function(m, na.char = '?'){
 # reshape2::melt(lapply(x, class))
 # showme(x) 
 fix.data <- function(x, 
-                     type = c("all", "names", "values", "types"), 
-                     source_chars = c(" ", ",", "/", "|", "#", "-", "(", ")"), 
+                     type = c("all", "names", "values", "types"),
+                     source_chars = c(" ", "'", ",", "/", "|", "#", 
+                                      "-", "(", ")", "[", "]", "{", "}"),
                      destination_char = "_", 
                      numeric_class = c("difftime"), 
                      nominal_class = c("factor", "logical", "Date", "POSIXct", "POSIXt"))
@@ -324,25 +446,29 @@ fix.data.types <- function(x,
 ###############################
 #prune.data
 ###############################
-prune.data <- function(data, mcfs_result, size = NA){
+prune.data <- function(x, mcfs_result, size = NA){
   
-  if(class(mcfs_result)!="mcfs")
-    stop("Input object is not 'mcfs' class.")
+  if(class(mcfs_result)!="mcfs"){
+    mystop("Input object is not 'mcfs' class.")
+    return(NULL)
+  }
   
   if(is.na(size))
     size <- mcfs_result$cutoff_value
   if(is.null(size) | is.na(size) | size <= 0){
-    warning(paste0("Parameter 'size' is NULL, NA or <= 0."))
+    mystop(paste0("Parameter 'size' is NULL, NA or <= 0."))
     return(NULL)
   }
   
-  fdata <- data[,names(data) %in% as.character(head(mcfs_result$RI,size)$attribute)]
-  target.data.frame <- data.frame(data[,mcfs_result$target])
-  if(ncol(target.data.frame)==1){
-    names(target.data.frame) <- mcfs_result$target
-    fdata <- cbind(fdata, target.data.frame)
-  }
-  return(fdata)
+  target <- attr(x, 'target')
+  if(is.null(target))
+    target <- mcfs_result$target
+  
+  pruned_data <- x[,names(x) %in% c(as.character(head(mcfs_result$RI$attribute,size)),target), drop = F]
+  attr(pruned_data, 'attr_weights') <- attr(x, 'attr_weights')[names(x) %in% c(as.character(head(mcfs_result$RI$attribute,size)),target)]
+  attr(pruned_data, 'target') <- attr(x, 'target')
+  
+  return(pruned_data)
 }
 
 ###############################
@@ -359,7 +485,8 @@ read.ID <- function(fileName){
         interdeps <- read.ID.list(fileName)
       }
   }else{
-    stop(paste0("File: '",fileName,"' does not exists."))
+    mystop(paste0("File: '",fileName,"' does not exists."))
+    return(NULL)
   }
   
   return(interdeps)
@@ -388,7 +515,7 @@ read.ID.list <- function(fileName) {
     return (NULL)
   if(is.na(d[1,2]))
     return (NULL)
-
+  
   d <- data.frame(edge_a=d[,1], edge_b=d[,2], weight=as.numeric(d[,3]), stringsAsFactors = FALSE)
   d <- d[order(-d[,3]),]
   d <- data.frame(position = 1:nrow(d), d)
@@ -435,26 +562,41 @@ read.RI <- function(fileName){
   ranking <- NULL
   if(File.exists(fileName)){
     ranking <- read.table(fileName, sep=",", header = TRUE, stringsAsFactors = FALSE)
-    ranking <- ranking[, names(ranking) %in% c('attribute', 'projections', 'classifiers', 'crudeRI', 'nodes', 'RI_norm')]
+    ranking <- ranking[, names(ranking) %in% c('attribute', 'projections', 'classifiers', 'crudeRI', 'nodes', 'RI', 'RI_norm')]
     # crudeRI is now nodes
     names(ranking)[names(ranking) %in% c('crudeRI')] <- 'nodes'
-    ranking <- ranking[order(-ranking$RI_norm),]
+    #if there is RI_norm remove RI and replace RI_norm by RI
+    if('RI_norm' %in% names(ranking)){
+      ranking <- ranking[, !names(ranking) %in% c('RI')]
+      names(ranking)[names(ranking) == 'RI_norm'] <- 'RI'
+    }
+    rownames(ranking) <- 1:nrow(ranking)
+    ranking <- ranking[order(-ranking$RI),]
     position <- 1:nrow(ranking)
-    ranking <- cbind(position,ranking)
+    ranking <- cbind(position, ranking)
   }else{
-    stop(paste0("File: '",fileName,"' does not exists."))
+    mystop(paste0("File: '",fileName,"' does not exists."))
+    return(NULL)
   }
     
   return (ranking)
 }
 
+###############################
+#write.RI
+###############################
+write.RI <- function(ri, fileName){
+  myRI <- ri[order(as.numeric(rownames(ri))),]
+  save.csv(myRI, file=fileName, row.names = F)
+}
+
 ###############################
 #read.csv.result
 ###############################
 read.csv.result <- function(fileName){
   resultDataFrame <- NULL
   if(File.exists(fileName))
-    resultDataFrame <- read.table(fileName, sep=",", header = TRUE, na.strings = c("NA", "?", "NaN"), stringsAsFactors = FALSE)
+    resultDataFrame <- load.csv(fileName, na.char = c('?', 'NA', 'NaN'))
   return (resultDataFrame)
 }
 
@@ -485,13 +627,23 @@ read.params <- function(fileName){
 ###############################
 #import.result
 ###############################
-import.result <- function(path = "./", label){
+import.result <- function(path = "./", label = NA){
+
+  if(!dir.exists(dirname(path))){
+    mystop(paste0("Path does not exist. Path: ", path))
+    return(NULL)
+  }
   
-  if(!dir.exists(file.path(path))){
-    stop(paste0("Path does not exist. Path: ", path))
+  if(tolower(file.ext(basename(path))) == 'zip'){
+    zip_file <- path
+    label <- drop.file.ext(basename(path))
+  }else if(is.na(label) || is.null(label)){
+    mystop(paste0("Result label is not defined."))
+    return(NULL)
+  }else{
+    zip_file <- file.path(path, paste0(label, '.zip'))
   }
   
-  zip_file <- file.path(path, paste0(label, '.zip'))
   if(File.exists(zip_file)){
     zip <- T
     tmp_dir <- tempdir()
@@ -508,8 +660,11 @@ import.result <- function(path = "./", label){
   if(!File.exists(id_file))
     id_file <- file.path(tmp_dir, paste0(label, "_ID.csv"))
   
+  ri_file_phase_1 <- file.path(tmp_dir, paste0(label, "__RI_phase_1.csv"))
+  ri_file_phase_2 <- file.path(tmp_dir, paste0(label, "__RI_phase_2.csv"))
   distances_file <- file.path(tmp_dir, paste0(label, "_distances.csv"))
   matrix_file <- file.path(tmp_dir, paste0(label, "_cmatrix.csv"))
+  matrix_top_file <- file.path(tmp_dir, paste0(label, "_cmatrix_top.csv"))
   cutoff_file <- file.path(tmp_dir, paste0(label, "_cutoff.csv"))
   cv_file <- file.path(tmp_dir, paste0(label, "_cv_accuracy.csv"))
   permutations_file <- file.path(tmp_dir, paste0(label, "_permutations.csv"))
@@ -521,26 +676,41 @@ import.result <- function(path = "./", label){
   params_file <- file.path(tmp_dir, paste0(label, ".run"))
   
   mcfsResult <- list()
+
   mcfsResult$data <- NULL
   if(File.exists(data_file)){
     mcfsResult$data <- read.adh(data_file)
   }else if(File.exists(data_csv_file)){
-    mcfsResult$data <- utils::read.csv(data_csv_file, header = T, sep=',', na.strings = c("NA", "NaN", "?") , stringsAsFactors = F)
+    mcfsResult$data <- load.csv(data_csv_file, na.char = c('?', 'NA', 'NaN'))
   }
-
+  
   params <- read.params(params_file)
   if(File.exists(matrix_file)){
     mcfsResult$target <- read.target(matrix_file)
   }else{
     mcfsResult$target <- params$target
   }
+  
   mcfsResult$RI <- read.RI(ri_file)
-
+  if(File.exists(ri_file_phase_1) & File.exists(ri_file_phase_2)){
+    mcfsResult$RI_mode_2 <- list()
+    mcfsResult$RI_mode_2$phase_1 <- read.RI(ri_file_phase_1)
+    mcfsResult$RI_mode_2$phase_2 <- read.RI(ri_file_phase_2)
+  }
+  
   if(File.exists(id_file)){
     mcfsResult$ID <- read.ID(id_file)
   }
   mcfsResult$distances <- read.csv.result(distances_file)
-  mcfsResult$cmatrix <- read.cmatrix(matrix_file)
+  
+  if(File.exists(matrix_file)){
+    mcfsResult$cmatrix <- list()
+    mcfsResult$cmatrix$rnd_features <- read.cmatrix(matrix_file)
+    if(File.exists(matrix_top_file)){
+      mcfsResult$cmatrix$top_features <- read.cmatrix(matrix_top_file)
+    }
+  }
+
   mcfsResult$cutoff <- read.csv.result(cutoff_file)
   topRanking <- read.csv.result(topRanking_file)
   if(is.null(topRanking)){
@@ -579,11 +749,21 @@ import.result <- function(path = "./", label){
 ###############################
 export.result <- function(mcfs_result, path = "./", label = "rmcfs", zip = TRUE){
 
-  if(class(mcfs_result) != "mcfs")
-    stop("Input object is not 'mcfs' class.")
+  if(class(mcfs_result) != "mcfs"){
+    mystop("Input object is not 'mcfs' class.")
+    return(NULL)
+  }
+  
+  if(tolower(file.ext(basename(path))) == 'zip'){
+    zip <- TRUE
+    label <- drop.file.ext(basename(path))
+    path <- dirname(path)
+  }
   
   #in any case create the directory if it does not exist
-  dir.create(file.path(path), showWarnings = F, recursive = T)
+  if(!dir.exists(path))
+    dir.create(path, showWarnings = F, recursive = T)
+    
   if(zip){
     tmp_dir <- tempdir()
   }else{
@@ -591,41 +771,58 @@ export.result <- function(mcfs_result, path = "./", label = "rmcfs", zip = TRUE)
   }
 
   ri_file <- file.path(tmp_dir, paste0(label, "__RI.csv"))
+  ri_file_phase_1 <- file.path(tmp_dir, paste0(label, "__RI_phase_1.csv"))
+  ri_file_phase_2 <- file.path(tmp_dir, paste0(label, "__RI_phase_2.csv"))
   id_file <- file.path(tmp_dir, paste0(label, "_ID.csv"))
   distances_file <- file.path(tmp_dir, paste0(label, "_distances.csv"))
   matrix_file <- file.path(tmp_dir, paste0(label, "_cmatrix.csv"))
+  matrix_top_file <- file.path(tmp_dir, paste0(label, "_cmatrix_top.csv"))
   cutoff_file <- file.path(tmp_dir, paste0(label, "_cutoff.csv"))
   cv_file <- file.path(tmp_dir, paste0(label, "_cv_accuracy.csv"))
   permutations_file <- file.path(tmp_dir, paste0(label, "_permutations.csv"))    
   topRanking_file <- file.path(tmp_dir, paste0(label, "_topRanking.csv"))
   jrip_file <- file.path(tmp_dir, paste0(label, "_jrip.txt"))
   predictionStats_file <- file.path(tmp_dir, paste0(label, "_predictionStats.csv"))
-  data_file <- file.path(tmp_dir, paste0(label, "_data.csv"))
+  data_file <- file.path(tmp_dir, paste0(label, "_data.adh"))
   params_file <- file.path(tmp_dir, paste0(label, ".run"))
-    
-  write.csv(mcfs_result$RI, file=ri_file, row.names = F)
-  write.csv(mcfs_result$ID, file=id_file, row.names = F)
-  write.csv(mcfs_result$distances, file=distances_file, row.names = F)
+  
+  write.RI(mcfs_result$RI, fileName=ri_file)
+  if(!is.null(mcfs_result$RI_mode_2$phase_1) & !is.null(mcfs_result$RI_mode_2$phase_2)){
+    write.RI(mcfs_result$RI_mode_2$phase_1, fileName=ri_file_phase_1)
+    write.RI(mcfs_result$RI_mode_2$phase_2, fileName=ri_file_phase_2)
+  }
+  
+  data.table::fwrite(as.data.table(mcfs_result$ID), file=id_file, row.names = F)
+  data.table::fwrite(as.data.table(mcfs_result$distances), file=distances_file, row.names = F)
+
   #save cmatrix
   if(any(names(mcfs_result)=="cmatrix")){
-    cmatrix <- as.data.frame(mcfs_result$cmatrix)
+    cmatrix <- as.data.frame(mcfs_result$cmatrix$rnd_features)
     cmatrix <- cbind(rownames(cmatrix),cmatrix)
     colnames(cmatrix)[1] <- mcfs_result$target
-    write.csv(cmatrix, file=matrix_file, row.names = F)
+    save.csv(cmatrix, file=matrix_file, row.names = F)
+    
+    if(!is.null(mcfs_result$cmatrix$top_features)){
+      cmatrix <- as.data.frame(mcfs_result$cmatrix$top_features)
+      cmatrix <- cbind(rownames(cmatrix),cmatrix)
+      colnames(cmatrix)[1] <- mcfs_result$target
+      save.csv(cmatrix, file=matrix_top_file, row.names = F)
+    }
   }
-  #save cmatrix
+  
+  #save predictionStats
   if(any(names(mcfs_result)=="predictionStats")){
-    write.csv(mcfs_result$predictionStats, file=predictionStats_file, row.names = F)
+    save.csv(mcfs_result$predictionStats, file=predictionStats_file, row.names = F)
   }
   #save cutoff
-  write.csv(mcfs_result$cutoff, file=cutoff_file, row.names = F)
+  save.csv(mcfs_result$cutoff, file=cutoff_file, row.names = F)
   #save cv_accuracy
   if(any(names(mcfs_result)=="cv_accuracy")){
-    write.csv(mcfs_result$cv_accuracy, file=cv_file, row.names = F)
+    save.csv(mcfs_result$cv_accuracy, file=cv_file, row.names = F)
   }    
   #save permutations
   if(any(names(mcfs_result)=="permutations")){
-    write.csv(mcfs_result$permutations, file=permutations_file, row.names = F)
+    save.csv(mcfs_result$permutations, file=permutations_file, row.names = F)
   }
   #save top ranking
   topRanking <- head(mcfs_result$RI, mcfs_result$cutoff_value)
@@ -633,7 +830,7 @@ export.result <- function(mcfs_result, path = "./", label = "rmcfs", zip = TRUE)
     position <- 1:nrow(topRanking)
     topRanking <- cbind(position, topRanking)
   }
-  write.csv(topRanking, file=topRanking_file, row.names = F)
+  save.csv(topRanking, file=topRanking_file, row.names = F)
   #save jrip rules
   if(any(names(mcfs_result)=="jrip")){
     writeChar(mcfs_result$jrip, jrip_file)
@@ -644,9 +841,10 @@ export.result <- function(mcfs_result, path = "./", label = "rmcfs", zip = TRUE)
   }
   #save data
   if(any(names(mcfs_result)=="data")){
-    write.csv(mcfs_result$data, file=data_file, row.names = F)
+    #save.csv(mcfs_result$data, file=data_file, row.names = F)
+    write.adh(mcfs_result$data, file=data_file, target = mcfs_result$target, zip = F)
   }
-  
+
   if(zip){
       zip_file <- file.path(path, paste0(label, '.zip'))
       tmp.files <- get.files.names(tmp_dir, filter=label, ext=c('.run','.csv','.txt','.adx','.adh'), fullNames=T, recursive=F)
@@ -688,38 +886,40 @@ artificial.data <- function(rnd_features = 500, size = c(40, 20, 10), corruption
 #build.idgraph
 ###############################
 build.idgraph <- function(mcfs_result, size = NA, size_ID = NA, 
-                          self_ID = FALSE, outer_ID = FALSE, orphan_nodes = FALSE, 
+                          self_ID = FALSE, outer_ID = FALSE, orphan_nodes = FALSE,
                           size_ID_mult = 3, size_ID_max = 100) {
 
-  if(class(mcfs_result)!="mcfs")
-    stop("Input object is not 'mcfs' class.")
+  if(class(mcfs_result)!="mcfs"){
+    mystop("Input object is not 'mcfs' class.")
+    return(NULL)
+  }
   
   if(all(names(mcfs_result)!="ID")){
-    stop("ID-Graph edges are not collected. Object 'mcfs_result$ID' does not exist.")
+    mystop("ID-Graph edges are not collected. Object 'mcfs_result$ID' does not exist.")
+    return(NULL)
   }
   
   if(is.na(size))
     size <- mcfs_result$cutoff_value
   if(is.null(size) | is.na(size) | size <= 0){
-    warning(paste0("Parameter 'size' is NULL, NA or <= 0."))
+    mystop(paste0("Parameter 'size' is NULL, NA or <= 0."))
+    return(NULL)
   }
   
   plot_minW <- 1
-  plot_maxW <- 7  
+  plot_maxW <- 5
   vertexMinSize <- 3
   vertexMaxSize <- 12
   
   #add weightNorm and color columns to ranking
-  ranking <- mcfs_result$RI
-  ranking$attribute <- as.character(ranking$attribute)
-  ranking$color <- scale.vector(ranking$RI_norm,0,1)
+  ranking <- mcfs_result$RI  
+  ranking$attribute <- as.character(ranking$attribute)  
+  ranking$color <- scale.vector(ranking$RI, 0, 1)
   ranking$color <- abs(ranking$color-1)
   
-  #add weightNorm and color columns to interdeps
-  #interdeps <- mcfs_result$ID
   interdeps <- mcfs_result$ID[!is.na(mcfs_result$ID$weight),]
-
-  if(is.null(interdeps)){
+  
+  if(is.null(interdeps) | nrow(interdeps)==0){
     warning("ID-Graph is empty. Change input parameters and try to build it again.")
     return (NULL)
   }
@@ -727,10 +927,10 @@ build.idgraph <- function(mcfs_result, size = NA, size_ID = NA,
   if(!self_ID)
     interdeps <- interdeps[interdeps$edge_a != interdeps$edge_b,]
   
-  interdeps$weightNorm <- scale.vector(interdeps$weight,plot_minW,plot_maxW)
-  interdeps$color <- scale.vector(interdeps$weight,0,1)
+  interdeps$weightNorm <- scale.vector(interdeps$weight, plot_minW, plot_maxW)
+  interdeps$color <- scale.vector(interdeps$weight, 0, 1)
   interdeps$color <- abs(interdeps$color-1)
-
+  
   #select interdeps to plot
   top_nodes <- head(ranking, size)
   #interdeps <- interdeps[is.element(interdeps$edge_a, top_nodes$attribute),]
@@ -740,12 +940,12 @@ build.idgraph <- function(mcfs_result, size = NA, size_ID = NA,
                              is.element(interdeps$edge_b, top_nodes$attribute),]
   }else{
     interdeps <- interdeps[is.element(interdeps$edge_a, top_nodes$attribute) & 
-                             is.element(interdeps$edge_b, top_nodes$attribute),]
+                           is.element(interdeps$edge_b, top_nodes$attribute),]
   }
   if(is.na(size_ID)){
-    size_ID <- min(size * size_ID_mult, size_ID_max, nrow(interdeps))
+    size_ID <- min(size * size_ID_mult, size_ID_max)
   }
-  #min_ID <- get.min.ID(mcfs_result, size, size_ID, size_ID_mult, size_ID_max)
+  size_ID <- min(size_ID, nrow(interdeps))
   min_ID <- sort(interdeps$weight, decreasing=T)[size_ID]
   interdeps <- interdeps[as.numeric(interdeps$weight) >= min_ID,]
   nodes_to_keep <- ranking[ranking$attribute %in% unique(c(top_nodes$attribute, interdeps$edge_a, interdeps$edge_b)),]
@@ -753,23 +953,28 @@ build.idgraph <- function(mcfs_result, size = NA, size_ID = NA,
   #select ranking to plot
   gNodes <- nodes_to_keep
   if(orphan_nodes == FALSE){
-    gNodes <- unique(c(interdeps$edge_a,interdeps$edge_b))
+    gNodes <- unique(c(interdeps$edge_a, interdeps$edge_b))
     gNodes <- nodes_to_keep[nodes_to_keep$attribute %in% gNodes,]
   }
   cat(paste0("Selected ",nrow(gNodes)," nodes and ", nrow(interdeps)," edges.\n"))
   g <- igraph::graph.empty()
-  if(nrow(interdeps) > 0) {    
-    #add nodes
-    for(i in 1:nrow(gNodes)){
-      c <- rgb(1, gNodes$color[i], gNodes$color[i])
-      g <- g + igraph::vertices(gNodes$attribute[i], shape="circle", color=c, size=10)
-    }
-    #add edges
-    for(i in 1:nrow(interdeps)){
-      c <- rgb(interdeps$color[i], interdeps$color[i], interdeps$color[i])
-      g <- g + igraph::edge(interdeps$edge_a[i],interdeps$edge_b[i], weight=interdeps$weightNorm[i],
-                            width=interdeps$weightNorm[i], color=c)
-    }
+  if(nrow(gNodes) == 0){
+    return(g)
+  }
+  
+  if(nrow(interdeps) > 0) {
+    #create nodes df
+    nodes_df <- data.frame(name = gNodes$attribute, color = rgb(1, gNodes$color, gNodes$color), shape = "circle", size = 10, sat = gNodes$color)
+    #print(nodes_df)
+
+    #create edges df
+    interdeps$color <- rgb(interdeps$color, interdeps$color, interdeps$color)
+    interdeps <- interdeps[,-1]
+    names(interdeps) <- c("from","to","weight","width","color")
+    #print(interdeps)
+    
+    #create graph based on nodes and edges data.frames
+    g <- igraph::graph_from_data_frame(interdeps, directed=TRUE, vertices=nodes_df)
     
     from <- NULL
     to <- NULL
@@ -782,15 +987,13 @@ build.idgraph <- function(mcfs_result, size = NA, size_ID = NA,
     vertexSizeFrom <- sapply(m, function(x) length(igraph::E(g)[from(x)]))
     vertexSizeTo <- sapply(m, function(x) length(igraph::E(g)[to(x)]))
     vertexSize <- vertexSizeFrom + vertexSizeTo
-
-    V(g)$size <- scale.vector(vertexSize,vertexMinSize,vertexMaxSize)
-    #vertex.attributes(g)$size <- scale.vector(vertexSize,vertexMinSize,vertexMaxSize)
+    #V(g)$edges <- vertexSize
+    V(g)$size <- scale.vector(vertexSize, vertexMinSize, vertexMaxSize)
   }
-  
+
   class(g) <- append("idgraph", class(g))
   return(g)  
 }
-
 ###############################
 #get.min.ID
 ###############################
@@ -825,14 +1028,16 @@ get.min.ID <- function(mcfs_result, size = NA, size_ID = NA, size_ID_mult = 3, s
 ###############################
 print.mcfs <- function(x, ...){
   mcfs_result <- x
-  if(class(mcfs_result)!="mcfs")
+  if(class(mcfs_result)!="mcfs"){
     stop("Input object is not 'mcfs' class.")
+    return(NULL)
+  }
 
   writeLines(paste0("##### MCFS-ID result (s = ", mcfs_result$params$mcfs.projections,", t = ",mcfs_result$params$mcfs.splits, ", m = ",mcfs_result$params$mcfs.projectionSize, ") #####"))
   writeLines(paste0("Target feature: '",mcfs_result$target,"'"))
   writeLines("")
   writeLines(paste0("Top ",mcfs_result$cutoff_value," features:"))
-  print(head(mcfs_result$RI[,c("position", "attribute", "RI_norm")], mcfs_result$cutoff_value), row.names = F)
+  print(head(mcfs_result$RI[,c("position", "attribute", "RI")], mcfs_result$cutoff_value), row.names = F)
   writeLines("")
   writeLines("#################################")
   writeLines("Cutoff values:")
@@ -841,7 +1046,7 @@ print.mcfs <- function(x, ...){
   if(any(names(mcfs_result) %in% c("cmatrix"))){
     writeLines("#################################")
     writeLines("Confusion matrix obtained on randomly selected (st) datasets:")
-    print.cmatrix(mcfs_result$cmatrix)
+    print.cmatrix(mcfs_result$cmatrix$rnd_features)
     writeLines("")
   }
   if(any(names(mcfs_result) %in% c("predictionStats"))){
