@@ -1,6 +1,6 @@
 /*******************************************************************************
  * #-------------------------------------------------------------------------------
- * # Copyright (c) 2003-2016 IPI PAN.
+ * # dmLab 2003-2019
  * # All rights reserved. This program and the accompanying materials
  * # are made available under the terms of the GNU Public License v3.0
  * # which accompanies this distribution, and is available at
@@ -15,16 +15,9 @@
  * # Algorithm 'SLIQ' developed by Mariusz Gromada
  * # R Package developed by Michal Draminski & Julian Zubek
  * #-------------------------------------------------------------------------------
- * # If you want to use dmLab or MCFS/MCFS-ID, please cite the following paper:
- * # M.Draminski, A.Rada-Iglesias, S.Enroth, C.Wadelius, J. Koronacki, J.Komorowski 
- * # "Monte Carlo feature selection for supervised classification", 
- * # BIOINFORMATICS 24(1): 110-117 (2008)
- * #-------------------------------------------------------------------------------
  *******************************************************************************/
 package dmLab.mcfs;
 
-import java.io.File;
-
 import dmLab.DMLabInfo;
 import dmLab.mcfs.mcfsEngine.MCFSExperiment;
 
@@ -35,8 +28,8 @@ public class MCFS
 	{		
 		DMLabInfo dmLabInfo = new DMLabInfo();      
 		System.out.println(dmLabInfo.toString());
-
-		String paramsFileName;      
+		
+		String paramsFileName;
 		if(args.length>=1){
 			paramsFileName=args[0];
 		}else{
@@ -44,15 +37,12 @@ public class MCFS
 			return;
 		}
 
-		MCFSParams mcfsParams = new MCFSParams();
-
-		File tmpDir = new File(MCFSParams.TMP_PATH);
-		if(!tmpDir.exists()) {
-			tmpDir.mkdir();
-		}
-
+		MCFSParams mcfsParams = new MCFSParams();		
 		if(mcfsParams.load("", paramsFileName)){
-			for(int i=0; i<mcfsParams.inputFiles.length; i++){
+			//MDR DEBUG
+			//mcfsParams.verbose = false;
+			//mcfsParams.tmpPATH = new File("/Users/mdraminski/TEMP1/").getAbsolutePath();
+			for(int i=0; i<mcfsParams.inputFiles.length; i++){				
 				mcfsParams.inputFileName = mcfsParams.inputFiles[i];
 				//System.out.println("Input File: " + mcfsParams.inputFileName);
 				MCFSExperiment mcfs = new MCFSExperiment(mcfsParams);
